//////////////////////////////////////////////////////////////////////
//////Grn2Red - Green blood to red blood converter for GBA Doom///////
//////////////////////By Samuel 'Kaiser' Villarreal///////////////////
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "bloodyTextures.h"

long G_LongSwap (long dat);
#define LSWAP(x) G_LongSwap (x)

typedef unsigned char byte;
typedef unsigned int uint;
typedef struct
{
  char identification[4];
  int  numlumps;
  int  infotableofs;
} wadinfo_t;
typedef struct
{
  int  filepos;
  int  size;
  char name[8];
} filelump_t;
typedef struct
{
	byte* data;
	char texture[9];
} bloodyTextures_t;

wadinfo_t wad;
filelump_t *lump;

static const char *bloodySprites[] = 
{
	"SARGG1",
	"SARGG2",
	"SARGG3",
	"SARGG4",
	"SARGG5",
	"SARGG6",
	"SARGG7",
	"SARGG8",
	"SARGI0",
	"SARGJ0",
	"SARGK0",
	"SARGL0",
	"SARGM0",
	"SARGN0",
	"TROOI0",
	"TROOJ0",
	"TROOK0",
	"TROOL0",
	"TROOM0",
	"TROON0",
	"TROOO0",
	"TROOP0",
	"TROOQ0",
	"TROOR0",
	"TROOS0",
	"TROOT0",
	"TROOU0",
	"PLAYH0",
	"PLAYI0",
	"PLAYJ0",
	"PLAYK0",
	"PLAYL0",
	"PLAYM0",
	"PLAYN0",
	"PLAYO0",
	"PLAYP0",
	"PLAYQ0",
	"PLAYR0",
	"PLAYS0",
	"PLAYT0",
	"PLAYU0",
	"PLAYV0",
	"PLAYW0",
	"POSSH0",
	"POSSI0",
	"POSSJ0",
	"POSSK0",
	"POSSL0",
	"POSSM0",
	"POSSN0",
	"POSSO0",
	"POSSP0",
	"POSSQ0",
	"POSSR0",
	"POSSS0",
	"POSST0",
	"POSSU0",
	"SPOSH0",
	"SPOSI0",
	"SPOSJ0",
	"SPOSK0",
	"SPOSL0",
	"SPOSM0",
	"SPOSN0",
	"SPOSO0",
	"SPOSP0",
	"SPOSQ0",
	"SPOSR0",
	"SPOSS0",
	"SPOST0",
	"SPOSU0",
	"POL5A0",
	"POL1A0",
	"POL2A0",
	"POL4A0",
	"BLUDA0",
	"BLUDB0",
	"BLUDC0",
	"END"
};

static bloodyTextures_t bloodyTextures[] =
{
	{t_brick02,"BRICK02"},
	{t_brick03,"BRICK03"},
	{t_skin01,"SKIN01"},
	{t_skin02,"SKIN02"},
	{t_skin03,"SKIN03"},
	{t_sp_dude4,"MARBLE03"},
	{NULL, "END"}
};

//______________________________________________________________________
//main
//______________________________________________________________________

int main(int argc, char* argv[])
{
	byte* data;
	int size, i, offset, k, p = 0;
	uint *chunk;
	byte redData;
	FILE* f;
	if(!argv[1])
	{
		printf("No File\nGrn2Red.exe useage - Grn2Red.exe <path of gbadoom rom>\nOutput is always on root\n(c)2006 5-26-06 Samuel Villarreal\n");
		return 0;
	}
	f = fopen(argv[1], "rb");
	fseek(f, 0, SEEK_END);
	size = ftell(f);
	fseek(f, 0, SEEK_SET);

	data = (byte*)malloc(sizeof(char)*size);
	fread(data, sizeof(char)*size, 1, f);

	chunk = (uint*)data;
	for(i = 0; i < size / 4; i++)
	{
		if(*chunk++ == 1145132873)	//IWAD
		{
			offset = i * 4;
			free(chunk);
			printf("Found IWAD at offset 0x%X\nReading wad..\n", offset);
			break;
		}
	}
	if(offset == 0)
	{
		printf("Error: IWAD in rom not found\n");
		exit(1);
	}

	fseek(f, offset, SEEK_SET);
	fread(&wad, sizeof(wadinfo_t), 1, f);

	fseek(f, offset + LSWAP(wad.infotableofs), SEEK_SET);
	lump = (filelump_t*)malloc(sizeof(filelump_t)*LSWAP(wad.numlumps));
	fread(lump, sizeof(filelump_t)*LSWAP(wad.numlumps), 1, f);

	//handle sprites
	do
	{
		for(i = 0; i < LSWAP(wad.numlumps); i++)
		{
			if(!strncmp(lump[i].name, bloodySprites[p], 8))
			{
				printf("Reading lump: %s\n", bloodySprites[p++]);
				for(k = 0; k < LSWAP(lump[i+1].size); k++)
				{
					if((data[offset + LSWAP(lump[i+1].filepos) + k] & 0x70) == 0x70)
					{
						redData = (0xB0 + (data[offset +
							LSWAP(lump[i+1].filepos) + k] ^ 0x70));
						data[offset + LSWAP(lump[i+1].filepos) + k] = (byte)redData;
					}
				}
				break;
			}
		}
	} while(strncmp(bloodySprites[p], "END", 8));

	p = 0;
	//handle textures
	do
	{
		for(i = 0; i < LSWAP(wad.numlumps); i++)
		{
			if(!strncmp(lump[i].name, bloodyTextures[p].texture, 8))
			{
				printf("Reading lump: %s\n", bloodyTextures[p].texture);
				for(k = 0; k < LSWAP(lump[i].size); k++)
				{
					data[offset + LSWAP(lump[i].filepos) + k] = 
						(byte)bloodyTextures[p].data[k];
				}
				p++;
				break;
			}
		}
	} while(strncmp(bloodyTextures[p].texture, "END", 8));

	FILE* fp = fopen("DOOM_BLOOD.GBA", "wb");
	fwrite(data, sizeof(byte)*size, 1, fp);
	fclose(fp);

	free(lump);
	free(data);
	fclose(f);
	return 0;
}

//______________________________________________________________________
//G_LongSwap
//______________________________________________________________________

long G_LongSwap (long dat)
{
	unsigned    a,b,c,d;
	
	a = (dat>>24)&0xFF; b = (dat>>16)&0xFF; c = (dat>>8)&0xFF; d = dat&0xFF;
	return (d<<24)+(c<<16)+(b<<8)+a;
}

